# Autor: Ingmar Stapel
# Date: 20221018
# Version 1.0
# Homepage: https://www.blogyourearth.com/category/foto-gps/photogrammetrie 
# This python origran controls the stepper motor and picture recording
# and is unsed in my photogrammetry project
# Many thanks going to Bert Koessler for the initial python code which I only adapted for my project
# URL: https://www.heimkino-praxis.de/leinwand-maskierung-schrittmotor-steuerung/

import RPi.GPIO as GPIO
import sys
import time
import os
from picamera import PiCamera
from datetime import datetime  
from datetime import timedelta 

GPIO.setmode(GPIO.BOARD)

#################################
### Set the object name here ####
obj_name = "name of the object"
#################################

# set the numer of steps for each move of the stepper motor
num_step = 150
# set the number after which the roundtrip did a 360 degree spin
round_trip = 48

# set the camera resolution
camera_resolution_w = 3280
camera_resolution_h = 2464

# variable to check if the live stream is active
stream_active = True

# TB6600 motor driver setup
DIR = 33
PUL = 35
ENA = 37

# set the pins default to high or low according the direction the stepper motor
# should move to
DIR_Left = GPIO.HIGH
DIR_Right = GPIO.LOW

# set the motor driver to active
ENA_Locked = GPIO.LOW
ENA_Released = GPIO.HIGH

# activate the GPIO pins as output pins
GPIO.setwarnings(False)
GPIO.setup(DIR, GPIO.OUT)
GPIO.setup(PUL, GPIO.OUT)
GPIO.setup(ENA, GPIO.OUT)

# Motor setup for a smoth movement
STEP_ANGLE = 1.8 # degree
RAMP_LENGTH = 600 # steps
MIN_RPM = 100
MAX_RPM = 200

# Frequenzberechnung
stepsPerRevolution = 360 / STEP_ANGLE

minFrequency = 1 / (MAX_RPM / 60 * stepsPerRevolution)
maxFrequency = 1 / (MIN_RPM / 60 * stepsPerRevolution)

rampSlope = (maxFrequency - minFrequency) / RAMP_LENGTH

def stop_stream():
    #publish.single(MQTT_PATH, "stream stopped", hostname=MQTT_SERVER)
    print("live stream will be stopped")
    os.system("sudo service mjpg-streamer.sh stop")    
    print("stream stopped")
def start_stream():
    #publish.single(MQTT_PATH, "stream started", hostname=MQTT_SERVER)
    print("live stream will be started")
    os.system("sudo service mjpg-streamer.sh start")  
    print("stream started")

#def getCurrentPosition ():
#
#    try:
#        f = open(".position", "r")
#        steps = int(f.read())
#        f.close()
#        return steps
#    except IOError:
#        return 0

#def setCurrentPosition (steps):
#
#    f = open(".position", "w+");
#    f.write(str(10))
#    f.close()

# dreht den Motor relativ um eine bestimmte Anzahl Schritte
def moveBy (steps):
    GPIO.output(ENA, ENA_Locked)

    currentFreqency = maxFrequency

    for i in range(abs(steps)):

        # Schritt ausfuehren
        GPIO.output(PUL, GPIO.HIGH)
        time.sleep(currentFreqency / 2)
        
        GPIO.output(PUL, GPIO.LOW)
        time.sleep(currentFreqency / 2)
        
        # Rampensteigung auf aktuelle Frequenz anwenden
        if (abs(steps) > 2 * RAMP_LENGTH):
            if (i < RAMP_LENGTH):
                currentFreqency -= rampSlope
            else:
                if (i > abs(steps) - RAMP_LENGTH):
                    currentFreqency += rampSlope
        else:
            if (i < abs(steps) / 2):
                currentFreqency -= rampSlope
            else:
                currentFreqency += rampSlope
    GPIO.output(ENA, ENA_Released)


# need to generate unique file names and folders
time_string = time.strftime("%Y%m%d%H%M%S")
work_directory = "/home/pi/workspace/" + obj_name

# Check whether the specified path exists or not
isExist = os.path.exists(work_directory)

if not isExist:
    # Create a new directory because it does not exist 
    os.makedirs(work_directory)
    print("The new directory is created! "+ work_directory)

stop_stream()
time.sleep(0.5)

# Load the PiCamera module as camera
print("Camera warum-up time... 2 sec")
camera = PiCamera()
camera.resolution = (camera_resolution_w, camera_resolution_h)
time.sleep(2) # Camera warm-up time


counter = 1
while counter <= round_trip:
    time_string = time.strftime("%Y-%m-%d %H:%M:%S")
    #print(time_string)
    print("Picture counter: " + str(counter))
    # record pictures
    # Each file need a unique file name
    camera.capture(str(work_directory) + '/' + obj_name + '_' + time_string + '.jpg')
    moveBy(num_step)
    counter = counter + 1

    time.sleep(1)

# Close the PiCamera module to be able to start the live stream
camera.close()

lst = os.listdir(str(work_directory)) # your directory path
number_files = len(lst)
print("Pictures in the folder "+ str(work_directory) + ": "+ str(number_files))
start_stream()

print("\n360 degree recording finished\nStarting live stream again...")

